from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='ProTaska-GPT',
    version='0.0.5',
    description='Unleash the Potential of Datasets with Intelligent Tasks, Tutorials, and Algorithm Recommendations.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Aman Priyanshu, Supriti Vijay',
    author_email='amanpriyanshusms2001@gmail.com',
    packages=find_packages(exclude=["notebooks", "docs"]),
    url='https://github.com/AmanPriyanshu/protaska-gpt',
    install_requires=[
        'langchain',
        'numpy',
        'pandas',
        'colorama',
        'wikipedia',
        'openai',
        'datasets',
        'tiktoken'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



from langchain import ConversationChain, LLMChain, PromptTemplate
from langchain.memory import ConversationSummaryBufferMemory 
from langchain.chat_models import ChatOpenAI
from langchain.agents import AgentType
from langchain.agents import initialize_agent
from langchain.utilities import WikipediaAPIWrapper
from langchain.tools import Tool

wikipedia = WikipediaAPIWrapper()
wikipedia.doc_content_chars_max = 3000

template = """ProTaska is a Data Science and Machine Learning expert.

As ProTaska your job is to read the description of a dataset and explore opportunities for data-science. You are to explain the user different forms of ML/DS tasks they can perform if the user hasn't input anything. Each task should be mentioned as points, with a TLDR description of what they may entail.

If the user asks questions you must follow it with responses related to their input query. Remember to be simple and eli5 in your nature of responses.

Human: {human_input}
Assistant:"""

class ChatBotWrapper:
    def __init__(self, openai_key, dataset_description, agent_verbose=True):
        self.openai_key = openai_key
        self.dataset_description = dataset_description
        self.prompt = PromptTemplate(
            input_variables=["human_input"], 
            template=template
        )
        self.llm = ChatOpenAI(model_name="gpt-3.5-turbo",openai_api_key=self.openai_key,temperature=0)
        self.chatgpt_chain = LLMChain(
            llm=self.llm,
            prompt=self.prompt, 
            verbose=False, 
        )
        self.first_output = self.chatgpt_chain.predict(human_input="Data Description:\n"+self.dataset_description+'\n')
        input_string = template.format(human_input="Data Description:\n"+self.dataset_description+'\n')
        self.memory = ConversationSummaryBufferMemory(llm=self.llm, max_token_limit=200)
        self.memory.save_context({"input": input_string}, {"output": self.first_output})
        tools = Tool.from_function(
            func=wikipedia.run,
            name="Wikipedia",
            description="useful for when you need information and sources from Wikipedia!"
        ),
        self.agent_chain = initialize_agent(tools=tools, 
            llm=self.llm, 
            agent=AgentType.STRUCTURED_CHAT_ZERO_SHOT_REACT_DESCRIPTION, 
            memory=self.memory,
            verbose=agent_verbose
        )
        self.second_output = self.agent_chain.run(self.first_output+"\n\nFind relevant sources from Wikipedia from the above techniques and advances. Also include some TLDRs in front of those links. Be specific to the ML techniques previously mentioned.")

    def __call__(self, human_input):
        output = self.agent_chain.run(human_input)
        return output

def main(openai_key, dataset_description):
    chat_bot = ChatBotWrapper(openai_key, dataset_description)
    print("ProTaska:\t", chat_bot.first_output)
    print("ProTaska-Source:\t", chat_bot.second_output)
    print()
    while True:
        human_input = input("Human:\t")
        print("ProTaska:\t", chat_bot(human_input))
        print()


import json
from datasets.arrow_dataset import Dataset
import pandas as pd
from PIL import Image
import os

def load_json(destination_path):
    with open(destination_path, 'r') as file:
        json_data = json.load(file)
    meta_data = map_json_types(json_data)
    data_sample = map_json_examples(json_data)
    json_data = str(data_sample)+'\n```\nMeta Data:\n```\n'+str(meta_data)+'\n```\n'
    return json_data

def file_loader(destination_path):
    extension = os.path.splitext(destination_path)[1].lower()

    if extension in ['.jpg', '.jpeg', '.png', '.gif']:
        return load_image(destination_path)
    elif extension == '.csv':
        return load_csv(destination_path)
    elif extension == '.tsv':
        return load_tsv(destination_path)
    elif extension == '.json':
        return load_json(destination_path)
    elif extension in ['.txt', '.md']:
        return load_text(destination_path)
    elif extension == '.parquet':
        return load_parquet(destination_path)
    elif extension == '.xlsx':
        return load_excel(destination_path)
    else:
        return "Cannot Read Yet"

def load_image(destination_path):
    try:
        img = Image.open(destination_path)
        return "Image Format Correct"
    except:
        return "Not an Image"

def load_csv(destination_path):
    data = pd.read_csv(destination_path)
    # Limit to 5 rows
    data = data.head(5)
    return data.to_dict()

def load_tsv(destination_path):
    data = pd.read_csv(destination_path, sep='\t')
    # Limit to 5 rows
    data = data.head(5)
    return data.to_dict()

def load_text(destination_path):
    with open(destination_path, 'r') as file:
        # Limit to the first 500 characters
        data = file.read(500)
    return data

def load_parquet(destination_path):
    data = pd.read_parquet(destination_path)
    # Limit to 5 rows
    data = data.head(5)
    return data.to_dict()

def load_excel(destination_path):
    data = pd.read_excel(destination_path)
    # Limit to 5 rows
    data = data.head(5)
    return data.to_dict()

def map_json_types(json_input, threshold=10):
    if isinstance(json_input, dict):
        result = {}
        if len(list(json_input.keys()))<threshold:
            for key, value in json_input.items():
                result[key] = map_json_types(value)
            return result
        else:
            return "<Dict\{with lots of keys and items>\}"
    elif isinstance(json_input, list):
        if len(json_input) > 0:
            first_element = json_input[0]
            if isinstance(first_element, list) or isinstance(first_element, dict):
                return f"<List[{map_json_types(first_element)}]>"
            else:
                return f"<List[{type(first_element).__name__}]>"
        else:
            return json_input
    elif isinstance(json_input, Dataset):
        return '<```\n'+json_input.__repr__()+'\n```\n>'
    else:
        return type(json_input).__name__

def map_json_examples(json_input, threshold=10):
    if isinstance(json_input, dict):
        result = {}
        if len(list(json_input.keys()))<threshold:
            for key, value in json_input.items():
                result[key] = map_json_examples(value)
            return result
        else:
            return "<Dict\{with lots of keys and items>\}"
    elif isinstance(json_input, list):
        if len(json_input) > 0:
            first_element = json_input[0]
            if isinstance(first_element, list) or isinstance(first_element, dict):
                return f"<List[{map_json_examples(first_element)}]>"
            else:
                return f"<List[{str([str(v)[:100] for v in json_input[:3]])}]>"
        else:
            return json_input
    elif isinstance(json_input, Dataset):
        json_input = json_input.to_dict()
        return map_json_examples(json_input)
    else:
        return str(json_input)[:100]

# if __name__ == '__main__':
#     parsed_json = {"data": {"train": [["aa", "bb"], ["aa", "bb"]], "test": 0, "val": {"a": 100, "b": "c", "d": [1], "e": 4}}}
#     result = map_json_types(parsed_json)
#     output = json.dumps(result, indent=4)
#     print(output)
#     result = map_json_examples(parsed_json)
#     output = json.dumps(result, indent=4)
#     print(output)


import argparse
from .data.loader import HuggingFaceDatasetImporter, LocalDatasetImporter, KaggleDatasetImporter
from langchain.chat_models import ChatOpenAI

def describe_dataset(openai_key, importer_type, destination_path, dataset_key=None,):
    # Process input based on the selected importer
    llm = ChatOpenAI(model_name="gpt-3.5-turbo", openai_api_key=openai_key, temperature=0)
    if importer_type == "HuggingFaceDatasetImporter":
        data_ingestor = HuggingFaceDatasetImporter()
        data_ingestor.import_dataset(dataset_key, destination_path)
        data_ingestor.walk_dataset(destination_path)
        out = data_ingestor.ingest(llm=llm)
        return f"Description: {out}"
    elif importer_type == "KaggleDatasetImporter":
        data_ingestor = KaggleDatasetImporter()
        data_ingestor.import_dataset(dataset_key, destination_path)
        data_ingestor.walk_dataset(destination_path)
        out = data_ingestor.ingest(llm=llm)
        return f"Description: {out}"
    elif importer_type == "LocalDatasetImporter":
        data_ingestor = LocalDatasetImporter()
        data_ingestor.walk_dataset(destination_path)
        out = data_ingestor.ingest(llm=llm)
        return f"Description: {out}"
    else:
        return "Invalid selection"

def main():
    parser = argparse.ArgumentParser(description="Menu options for processing input.")
    parser.add_argument("openai_key", type=str, help="OpenAI Key")
    parser.add_argument("importer_type", type=str, choices=["HuggingFaceDatasetImporter", "KaggleDatasetImporter", "LocalDatasetImporter"], help="Importer Type")
    parser.add_argument("--dataset_key", type=str, help="Dataset Key (Optional)")
    parser.add_argument("destination_path", type=str, help="Destination Path")
    args = parser.parse_args()

    output = describe_dataset(args.openai_key, args.importer_type, args.destination_path, args.dataset_key)
    print("Output:", output)

if __name__ == "__main__":
    main()